"use strict";
/**
 * link-dependency-executables
 * Link a package's executables into node_modules
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.linkDependencyExecutables = void 0;
const debug_1 = __importDefault(require("debug"));
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
const E = __importStar(require("fp-ts/Either"));
const R = __importStar(require("fp-ts/Record"));
const TE = __importStar(require("fp-ts/TaskEither"));
const Apply = __importStar(require("fp-ts/Apply"));
const PathReporter = __importStar(require("io-ts/lib/PathReporter"));
const function_1 = require("fp-ts/function");
const PackageJsonBin_1 = require("@typescript-tools/io-ts/dist/lib/PackageJsonBin");
const monorepo_root_1 = require("@typescript-tools/monorepo-root");
const find_package_1 = require("@typescript-tools/find-package");
const StringifiedJSON_1 = require("@typescript-tools/io-ts/dist/lib/StringifiedJSON");
const debug = {
    cmd: debug_1.default('link')
};
const err = function_1.identity;
const findPackage = (packagePathOrName) => function_1.pipe(
// FIXME: this should be exposed as a parameter
find_package_1.findPackage(process.cwd(), packagePathOrName), TE.mapLeft(err));
const monorepoRoot = function_1.flow(monorepo_root_1.monorepoRoot, E.mapLeft(err), TE.fromEither);
const packageJson = (packageDirectory) => path.resolve(packageDirectory, 'package.json');
const topLevelDependency = (monorepoRoot) => (dependency) => path.resolve(monorepoRoot, 'node_modules', dependency);
const readFile = (filename) => TE.tryCatch(() => __awaiter(void 0, void 0, void 0, function* () {
    return new Promise((resolve, reject) => fs.readFile(filename, 'utf8', (error, data) => error !== null && error !== undefined ? reject(error) : resolve(data)));
}), function_1.flow(E.toError, error => err({ type: 'unable to read file', filename, error })));
const decodePackageJsonBin = (dependency) => function_1.flow(StringifiedJSON_1.StringifiedJSON(PackageJsonBin_1.PackageJsonBin).decode.bind(null), E.mapLeft(function_1.flow(errors => PathReporter.failure(errors).join('\n'), error => err({ type: 'unable to decode dependency package.json', dependency, error }))), TE.fromEither);
const externalPackageManifest = (monorepoRoot) => (dependencyName) => function_1.pipe(packageJson(topLevelDependency(monorepoRoot)(dependencyName)), readFile, TE.chain(decodePackageJsonBin(dependencyName)));
const mkdir = (target) => function_1.pipe(TE.tryCatch(() => __awaiter(void 0, void 0, void 0, function* () {
    return new Promise((resolve, reject) => fs.mkdir(target, { recursive: true }, error => error !== null && error !== undefined ? reject(error) : resolve(function_1.constVoid())));
}), function_1.flow(E.toError, error => err({ type: 'unable to create directory', path: target, error }))));
/**
 * Note: defaults to no action when `target` already exists.
 * Wonder if I should instead remove the existing file before creating `target`.
 */
const symlink = (target, link) => function_1.pipe(mkdir(path.dirname(link)), TE.chain(() => function_1.pipe(TE.tryCatch(() => __awaiter(void 0, void 0, void 0, function* () {
    return new Promise((resolve, reject) => fs.symlink(target, link, error => error !== null && error !== undefined ? reject(error) : resolve(function_1.constVoid())));
}), E.toError), 
// recover from symlink-already-exists error
TE.orElse(error => error.message.startsWith('EEXIST:') ? TE.right(function_1.constVoid()) : TE.left(error)), TE.mapLeft(error => err({ type: 'unable to create symlink', target, path: link, error })))));
const symlinkExecutable = (monorepoRoot, internalPackageDirectory, dependencyName) => (executableName, targetPath) => {
    const symlinkLocation = path.resolve(internalPackageDirectory, 'node_modules', '.bin', executableName);
    const target = path.relative(path.dirname(symlinkLocation), path.resolve(topLevelDependency(monorepoRoot)(dependencyName), targetPath));
    return function_1.pipe(() => debug.cmd(`Creating symlink from ${symlinkLocation} -> ${target}`), TE.fromIO, TE.chain(() => symlink(target, symlinkLocation)));
};
const linkDependencyExecutables = (internalPackagePathOrName) => (dependencyName) => function_1.pipe(TE.bindTo('monorepoRoot')(monorepoRoot()), TE.bind('internalPackage', () => findPackage(internalPackagePathOrName)), TE.bind('dependencyManifest', ({ monorepoRoot }) => externalPackageManifest(monorepoRoot)(dependencyName)), TE.chain(({ monorepoRoot, internalPackage, dependencyManifest }) => {
    var _a;
    return function_1.pipe(((_a = dependencyManifest.bin) !== null && _a !== void 0 ? _a : {}), R.mapWithIndex(symlinkExecutable(monorepoRoot, internalPackage.location, dependencyName)), value => value, Apply.sequenceS(TE.taskEither));
}), TE.map(function_1.constVoid));
exports.linkDependencyExecutables = linkDependencyExecutables;
//# sourceMappingURL=index.js.map